@@ -1,6 +1,6 @@
 <?php
 
-#$CSK: gpsd.php,v 1.39 2006/11/21 22:31:10 ckuethe Exp $
+# DO NOT HAND_HACK! THIS FILE IS GENERATED
 
 # Copyright (c) 2006,2010 Chris Kuethe <chris.kuethe@gmail.com>
 #
@@ -16,7 +16,10 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 # OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-global $head, $blurb, $title, $googlemap, $autorefresh, $footer, $gmap_key;
+# Changed to Google Maps API v3, requires no API key, and shorter code
+# Sanjeev Gupta <ghane0@gmail.com> 2013-01-05
+
+global $head, $blurb, $title, $showmap, $autorefresh, $footer, $gmap_key;
 global $server, $advertise, $port, $open, $swap_ew, $testmode;
 $testmode = 1; # leave this set to 1
 
@@ -46,13 +49,13 @@ require_once("gpsd_config.inc");
 
 # sample data
 $resp = <<<EOF
-{"class":"POLL","timestamp":1270517274.846,"active":1,
- "fixes":[{"class":"TPV","tag":"MID41","device":"/dev/ttyUSB0",
+{"class":"POLL","time":"2010-04-05T21:27:54.84Z","active":1,
+ "tpv":[{"class":"TPV","tag":"MID41","device":"/dev/ttyUSB0",
            "time":1270517264.240,"ept":0.005,"lat":40.035093060,
            "lon":-75.519748733,"alt":31.1,"track":99.4319,
            "speed":0.123,"mode":3}],
- "skyviews":[{"class":"SKY","tag":"MID41","device":"/dev/ttyUSB0",
-              "time":1270517264.240,"hdop":9.20,"vdop":12.1,
+ "sky":[{"class":"SKY","tag":"MID41","device":"/dev/ttyUSB0",
+              "time":"2010-04-05T21:27:44.84Z","hdop":9.20,"vdop":12.1,
               "satellites":[{"PRN":16,"el":55,"az":42,"ss":36,"used":true},
                             {"PRN":19,"el":25,"az":177,"ss":0,"used":false},
                             {"PRN":7,"el":13,"az":295,"ss":0,"used":false},
@@ -89,9 +92,12 @@ if (isset($_GET['imgdata']) && $op == 'view'){
 
 	if ($testmode){
 		$sock = @fsockopen($server, $port, $errno, $errstr, 2);
+		@fwrite($sock, "?WATCH={\"enable\":true}\n");
+		usleep(1000);
 		@fwrite($sock, "?POLL;\n");
+		usleep(1000);
 		for($tries = 0; $tries < 10; $tries++){
-			$resp = @fread($sock, 1536); # SKY can be pretty big
+			$resp = @fread($sock, 2000); # SKY can be pretty big
 			if (preg_match('/{"class":"POLL".+}/i', $resp, $m)){
 				$resp = $m[0];
 				break;
@@ -113,6 +119,16 @@ else
 exit(0);
 
 ###########################################################################
+# Function to decide if a PRN is a true GPS bird or SBAS, GBAS, etc.
+# Sanjeev Gupta <ghane0@gmail.com> 20150408
+# Please refer to gps.h lines ~~ 95 , for a central definition
+function isGPS($PRN) {
+	if ($PRN <= 32) return TRUE ; 			# Navstar GPS
+	if ($PRN >= 64 && $PRN <= 96) return TRUE ; 	# GLONASS
+	if ($PRN >= 159 ) return TRUE ; 		# BeiDou ?
+	return FALSE ;					# SBAS, GBAS, unknown
+	}
+
 function colorsetup($im){
 	$C['white']	= imageColorAllocate($im, 255, 255, 255);
 	$C['ltgray']	= imageColorAllocate($im, 191, 191, 191);
@@ -166,7 +182,7 @@ function radial($angle, $sz){
 
 function azel2xy($az, $el, $sz){
 	global $swap_ew;
-	#rotate coords... 90deg W = 180deg trig
+	#rotate coords... 90deg E = 180deg trig
 	$az += 270;
 
 	#turn into radians
@@ -179,14 +195,14 @@ function azel2xy($az, $el, $sz){
 	# and convert length/azimuth to cartesian
 	$x = sprintf("%d", (($sz * 0.5) + ($r * cos($az))));
 	$y = sprintf("%d", (($sz * 0.5) + ($r * sin($az))));
-	if ($swap_ew == 0)
+	if ($swap_ew != 0)
 		$x = $sz - $x;
 
 	return array($x, $y);
 }
 
 function splot($im, $sz, $C, $e){
-	if ((0 == $e['PRN']) || (0 == $e['az'] + $e['el'] + $e['ss']) ||
+	if ((0 == $e['PRN']) || (0 == $e['az'] + $e['el']) ||
 	    ($e['az'] < 0) || ($e['el'] < 0))
 		return;
 
@@ -209,18 +225,19 @@ function splot($im, $sz, $C, $e){
 		$r = 8;
 
 	imageString($im, 3, $x+4, $y+4, $e['PRN'], $C['black']);
-	if ($e['used'] == true)
-		if ($e['PRN'] > 32)
-			imageFilledDiamond($im, $x, $y, $r, $color);
-		else
+        if ($e['used'] == true)
+		if (isGPS($e['PRN']))
 			imageFilledArc($im, $x, $y, $r, $r, 0, 360, $color, 0);
-	else
-		if ($e['PRN'] > 32)
-			imageDiamond($im, $x, $y, $r, $color);
 		else
+			imageFilledDiamond($im, $x, $y, $r, $color);
+	else
+		if (isGPS($e['PRN']))
 			imageArc($im, $x, $y, $r, $r, 0, 360, $color);
+		else
+			imageDiamond($im, $x, $y, $r, $color);
 }
 
+
 function imageDiamond($im, $x, $y, $r, $color){
 	$t = $r/2;
 	# this lunacy is because imagesetthickness doesn't seem to work
@@ -241,7 +258,7 @@ function imageFilledDiamond($im, $x, $y, $r, $color){
 		imagepolygon($im, $vx, 4, $color);
 		$t -= 0.5;
 	}
-}       
+}
 
 function elevation($im, $sz, $C, $a){
 	$b = 90 - $a;
@@ -280,7 +297,7 @@ function skyview($im, $sz, $C){
 
 	imageString($im, 4, $sz/2 + 4, 2        , 'N', $C['black']);
 	imageString($im, 4, $sz/2 + 4, $sz - 16 , 'S', $C['black']);
-	if ($swap_ew == 0){
+	if ($swap_ew != 0){
 		imageString($im, 4, 4        , $sz/2 + 4, 'E', $C['black']);
 		imageString($im, 4, $sz - 10 , $sz/2 + 4, 'W', $C['black']);
 	} else {
@@ -305,8 +322,8 @@ function gen_image($resp){
 	if ($sz > 240)
 		legend($im, $sz, $C);
 
-	for($i = 0; $i < count($GPS['skyviews'][0]['satellites']); $i++){
-		splot($im, $sz, $C, $GPS['skyviews'][0]['satellites'][$i]);
+	for($i = 0; $i < count($GPS['sky'][0]['satellites']); $i++){
+		splot($im, $sz, $C, $GPS['sky'][0]['satellites'][$i]);
 	}
 
 	header("Content-type: image/png");
@@ -325,7 +342,7 @@ function dfix($x, $y, $z){
 
 function write_html($resp){
 	global $sock, $errstr, $errno, $server, $port, $head, $body, $open;
-	global $blurb, $title, $autorefresh, $googlemap, $gmap_key, $footer;
+	global $blurb, $title, $autorefresh, $showmap, $gmap_key, $footer;
 	global $testmode, $advertise;
 
 	$GPS = json_decode($resp, true);
@@ -336,8 +353,8 @@ function write_html($resp){
 	header("Content-type: text/html; charset=UTF-8");
 
 	global $lat, $lon;
-	$lat = (float)$GPS['fixes'][0]['lat'];
-	$lon = (float)$GPS['fixes'][0]['lon'];
+	$lat = (float)$GPS['tpv'][0]['lat'];
+	$lon = (float)$GPS['tpv'][0]['lon'];
 	$x = $server; $y = $port;
 	$imgdata = base64_encode($resp);
 	$server = $x; $port = $y;
@@ -347,20 +364,24 @@ function write_html($resp){
 	else
 		$autorefresh = '';
 
-	$gmap_head = $gmap_body = $gmap_code = '';
-	if ($googlemap){
-		$gmap_head = gen_gmap_head();
-		$gmap_body = 'onload="Load()" onunload="GUnload()"';
-		$gmap_code = gen_gmap_code();
+	$map_head = $map_body = $map_code = '';
+	if ($showmap == 1) {
+		$map_head = gen_gmap_head();
+		$map_body = 'onload="Load()" onunload="GUnload()"';
+		$map_code = gen_map_code();
+	} else if ($showmap == 2) {
+		$map_head = gen_osm_head();
+		$map_body = 'onload="Load()"';
+		$map_code = gen_map_code();
 	}
 	$part1 = <<<EOF
 <?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
-    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-<html xmlns="http://www.w3.org/1999/xhtml">
+    "https://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+<html xmlns="https://www.w3.org/1999/xhtml">
 <head>
 {$head}
-{$gmap_head}
+{$map_head}
 <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
 <meta http-equiv="Content-Language" content="en,en-us"/>
 <title>{$title} - GPSD Test Station {$lat}, {$lon}</title>
@@ -368,25 +389,25 @@ function write_html($resp){
 <style>
 .warning {
     color: #FF0000;
- }
+}
 
 .fixed {
-    font-family: courier, fixed;
+    font-family: mono-space;
 }
 
 .caption {
-    text-align: left; 
+    text-align: left;
     margin: 1ex 3em 1ex 3em; /* top right bottom left */
 }
 
 .administrivia {
-    font-size: small; 
+    font-size: small;
     font-family: verdana, sans-serif;
 }
 </style>
 </head>
 
-<body {$body} {$gmap_body}>
+<body {$body} {$map_body}>
 <center>
 <table border="0">
 <tr><td align="justify">
@@ -408,9 +429,9 @@ EOF;
 width="600" height="600"/>
 <br clear="all"/>
 <p class="caption">A filled circle means the satellite was used in
-the last fix. Green-yellow-red colors indicate signal strength in dB, 
- green=most and red=least.  Diamonds indicate SBAS satellites.</p>
-{$gmap_code}</td>
+the last fix. Green-yellow-red colors indicate signal strength in dB,
+green=most and red=least.  Diamonds indicate Augmentation satellites.</p>
+{$map_code}</td>
 </tr>
 EOF;
 
@@ -434,41 +455,61 @@ Use a different server:<br/>
 EOF;
 
 	if ($testmode && !$sock)
-		$part4 = "<tr><td><font color='red'>The gpsd instance that this page monitors is not running.</font></td></tr>";
+		$part4 = "<tr><td class='warning'>The gpsd instance that this page monitors is not running.</td></tr>";
 	else {
-		$nsv = count($GPS['skyviews'][0]['satellites']);
-		$ts = gmdate("r", $GPS['fixes'][0]['time']);
+		$fix = $GPS['tpv'][0];
+		$sky = $GPS['sky'][0];
+		$sats = $sky['satellites'];
+
+		$fixtype = array('Unknown' => 0, 'No Fix' => 1,'2D Fix' => 2, '3D Fix' => 3);
+		$type = array_search($fix['mode'],$fixtype);
+
+		$nsv = count($sats);
+                $ts = $fix['time'];
+                $sat = '';
+                foreach($sats as $s)
+                        $sat .= sprintf(
+                                "\t<tr><td>%d</td><td>%d</td><td>%d</td><td>%d</td><td>%s</td></tr>\n",
+                                $s['PRN'], $s['el'], $s['az'], $s['ss'], $s['used'] ? 'Y' : 'N'
+                        );
 		$part4 = <<<EOF
 <!-- ------------------------------------------------------------ -->
-        <tr><td align=center valign=top>
-	<table border=1>
-	<tr><td colspan=2 align=center><b>Current Information</b></td></tr>
-	<tr><td>Time (UTC)</td><td>{$ts}</td></tr>
-	<tr><td>Latitude</td><td>{$GPS['fixes'][0]['lat']}</td></tr>
-	<tr><td>Longitude</td><td>{$GPS['fixes'][0]['lon']}</td></tr>
-	<tr><td>Altitude</td><td>{$GPS['fixes'][0]['alt']}</td></tr>
-	<tr><td>Fix Type</td><td>{$GPS['fixes'][0]['mode']}</td></tr>
-	<tr><td>Satellites</td><td>{$nsv}</td></tr>
-	<tr><td>HDOP</td><td>{$GPS['skyviews'][0]['hdop']}</td></tr>
-	</table>
-</tr>
-<!-- raw response
+<tr><td align=center valign=top>
+    <table border=1>
+        <tr><th colspan=2 align=center>Current Information</th></tr>
+        <tr><td>Time (UTC)</td><td>{$ts}</td></tr>
+        <tr><td>Latitude</td><td>{$fix['lat']}</td></tr>
+        <tr><td>Longitude</td><td>{$fix['lon']}</td></tr>
+        <tr><td>Altitude</td><td>{$fix['alt']}</td></tr>
+        <tr><td>Fix Type</td><td>{$type}</td></tr>
+        <tr><td>Satellites</td><td>{$nsv}</td></tr>
+        <tr><td>HDOP</td><td>{$sky['hdop']}</td></tr>
+        <tr><td>VDOP</td><td>{$sky['vdop']}</td></tr>
+    </table>
+    <br/>
+    <table border=1>
+        <tr><th colspan=5 align=center>Current Satellites</th></tr>
+        <tr><th>PRN</th><th>Elevation</th><th>Azimuth</th><th>SS</th><th>Used</th></tr>
+$sat    </table>
+</td></tr>
+
+<!-- raw response:
 {$resp}
 -->
 EOF;
 	}
 
 	$part5 = <<<EOF
+
 </table>
 </center>
-
 {$footer}
-
 <hr/>
-<span class="administrivia">This script is distributed by the <a href="http://gpsd.berlios.de">GPSD project</a>.</span><br/>
-</body>
-</body>
+<p class="administrivia">This script is distributed by the
+<a href="https://gpsd.io/">GPSD project</a>.</p>
 
+</body>
+</html>
 EOF;
 
 print $part1 . $part2 . $part3 . $part4 . $part5;
@@ -495,9 +536,9 @@ function write_config(){
 #\$advertise = 'localhost';
 \$port = 2947;
 \$autorefresh = 0; # number of seconds after which to refresh
-\$googlemap = 0; # set to 1 if you want to have a google map
+\$showmap = 0; # set to 1 if you want to have a google map, set it to 2 if you want a map based on openstreetmap
 \$gmap_key = 'GetYourOwnGoogleKey'; # your google API key goes here
-\$swap_ew = 0; # set to 1 if you don't understand projections
+\$swap_ew = 0; # set to 1 for upward facing view (nonstandard)
 \$open = 0; # set to 1 to show the form to change the GPSd server
 
 ## You can read the header, footer and blurb from a file...
@@ -512,7 +553,7 @@ function write_config(){
 \$footer = '';
 \$blurb = <<<EOT
 This is a
-<a href="http://gpsd.berlios.de">gpsd</a>
+<a href="https://gpsd.io/">gpsd</a>
 server <blink><font color="red">located someplace</font></blink>.
 
 The hardware is a
@@ -532,50 +573,73 @@ EOB;
 function gen_gmap_head() {
 global $gmap_key;
 return <<<EOT
-<script src="http://maps.google.com/maps?file=api&amp;v=2&amp;key={$gmap_key}" type="text/javascript"></script>
-<script type="text/javascript">
-    <!--
-    // Create a base icon for all of our markers that specifies the shadow, icon
-    // dimensions, etc.
-function Load() {
-  if (GBrowserIsCompatible()) {
-    var map = new GMap2(document.getElementById("map"));
-    var point = new GLatLng( {$GLOBALS['lat']}, {$GLOBALS['lon']} );
-    map.setCenter( point, 14);
-    map.addControl(new GLargeMapControl());
-    map.addControl(new GMapTypeControl());
+<script src="//maps.googleapis.com/maps/api/js?sensor=false"/>
 
-    var baseIcon = new GIcon();
-    baseIcon.shadow = "http://www.google.com/mapfiles/shadow50.png";
-    baseIcon.iconSize = new GSize(20, 34);
-    baseIcon.shadowSize = new GSize(37, 34);
-    baseIcon.iconAnchor = new GPoint(9, 34);
-    baseIcon.infoWindowAnchor = new GPoint(9, 2);
-    baseIcon.infoShadowAnchor = new GPoint(18, 25);
+<script>
+ <!--
+    function Load() {
+      var map = new google.maps.Map(
+        document.getElementById('map'), {
+          center: new google.maps.LatLng({$GLOBALS['lat']}, {$GLOBALS['lon']}),
+          zoom: 13,
+          mapTypeId: google.maps.MapTypeId.ROADMAP
+      });
 
-    var icon = new GIcon(baseIcon);
-    icon.image = "http://www.google.com/mapfiles/marker.png";
-    var marker = new GMarker(point, icon);
-    map.addOverlay(marker);
-  }
+      var marker = new google.maps.Marker({
+            position: new google.maps.LatLng({$GLOBALS['lat']}, {$GLOBALS['lon']}),
+            map: map
+      });
+
+    }
+    google.maps.event.addDomListener(window, 'load', initialize);
+
+ -->
+</script>
+EOT;
 }
 
+function gen_osm_head() {
+global $GPS;
+return <<<EOT
+<script src="http://openlayers.org/api/OpenLayers.js" />
+<script>
+    <!--
+function Load() {
+	document.getElementById("map").firstChild.data = "";
+	var map = new OpenLayers.Map("map", {
+		controls: [
+			new OpenLayers.Control.Navigation(),
+			new OpenLayers.Control.PanZoomBar(),
+			new OpenLayers.Control.ScaleLine(),
+			new OpenLayers.Control.LayerSwitcher()
+		]
+	});
+	var layer = new OpenLayers.Layer.OSM("Open Street Map");
+	map.addLayer(layer);
+
+	var center = new OpenLayers.LonLat({$GLOBALS['lon']}, {$GLOBALS['lat']})
+		.transform(new OpenLayers.Projection("EPSG:4326"), map.getProjectionObject());
+	map.setCenter(center, 12);
+
+	var markers = new OpenLayers.Layer.Markers("Markers");
+	markers.addMarker(new OpenLayers.Marker(center));
+	map.addLayer(markers);
+}
     -->
     </script>
 EOT;
-
 }
-function gen_gmap_code() {
+
+function gen_map_code() {
 return <<<EOT
 <br/>
-    <div id="map" style="width: 550px; height: 400px; border:1px; border-style: solid;">
+<div id="map" style="width: 550px; height: 400px; border:1px; border-style: solid;">
     Loading...
     <noscript>
-<span class='warning'>Sorry: you must enable javascript to view our maps.</span><br/>
+        <span class='warning'>Sorry: you must enable javascript to view our maps.</span><br/>
     </noscript>
 </div>
 
-
 EOT;
 }
 
